@@ -8,6 +8,7 @@ import (
 	"net/http"
 	"net/url"
 	"os"
+	"path"
 	"regexp"
 	"sort"
 	"strings"
@@ -17,6 +18,7 @@ import (
 	"github.com/danielgtaylor/shorthand/v2"
 	"github.com/gosimple/slug"
 	"github.com/pb33f/libopenapi"
+	"github.com/pb33f/libopenapi/datamodel"
 	"github.com/pb33f/libopenapi/datamodel/high/base"
 	v3 "github.com/pb33f/libopenapi/datamodel/high/v3"
 	"github.com/pb33f/libopenapi/resolver"
@@ -26,7 +28,7 @@ import (
 )
 
 // reOpenAPI3 is a regex used to detect OpenAPI files from their contents.
-var reOpenAPI3 = regexp.MustCompile(`['"]?openapi['"]?:\s*['"]?3`)
+var reOpenAPI3 = regexp.MustCompile(`['"]?openapi['"]?\s*:\s*['"]?3`)
 
 // OpenAPI Extensions
 const (
@@ -59,6 +61,7 @@ type autoConfig struct {
 
 // Resolver is able to resolve relative URIs against a base.
 type Resolver interface {
+	GetBase() *url.URL
 	Resolve(uri string) (*url.URL, error)
 }
 
@@ -104,7 +107,7 @@ func getBasePath(location *url.URL, servers []*v3.Server) (string, error) {
 	prefix := fmt.Sprintf("%s://%s", location.Scheme, location.Host)
 
 	for _, s := range servers {
-		// Interprete all operation paths as relative to the provided location
+		// Interpret all operation paths as relative to the provided location
 		if strings.HasPrefix(s.URL, "/") {
 			return s.URL, nil
 		}
@@ -149,7 +152,7 @@ func getBasePath(location *url.URL, servers []*v3.Server) (string, error) {
 			}
 		}
 	}
-	return "", nil
+	return location.Path, nil
 }
 
 func getRequestInfo(op *v3.Operation) (string, *base.Schema, []interface{}) {
@@ -198,8 +201,19 @@ func getRequestInfo(op *v3.Operation) (string, *base.Schema, []interface{}) {
 	return "", nil, nil
 }
 
+// paramSchema returns a rendered schema line for a given parameter, falling
+// back to the param type info if no schema is available.
+func paramSchema(p *cli.Param, s *base.Schema) string {
+	schemaDesc := fmt.Sprintf("(%s): %s", p.Type, p.Description)
+	if s != nil {
+		schemaDesc = renderSchema(s, "  ", modeWrite)
+	}
+	return schemaDesc
+}
+
 func openapiOperation(cmd *cobra.Command, method string, uriTemplate *url.URL, path *v3.PathItem, op *v3.Operation) cli.Operation {
 	var pathParams, queryParams, headerParams []*cli.Param
+	var pathSchemas, querySchemas, headerSchemas []*base.Schema = []*base.Schema{}, []*base.Schema{}, []*base.Schema{}
 
 	// Combine path and operation parameters, with operation params having
 	// precedence when there are name conflicts.
@@ -224,8 +238,10 @@ func openapiOperation(cmd *cobra.Command, method string, uriTemplate *url.URL, p
 		var example interface{}
 
 		typ := "string"
+		var schema *base.Schema
 		if p.Schema != nil && p.Schema.Schema() != nil {
 			s := p.Schema.Schema()
+			schema = s
 			if len(s.Type) > 0 {
 				// TODO: support params of multiple types?
 				typ = s.Type[0]
@@ -276,25 +292,31 @@ func openapiOperation(cmd *cobra.Command, method string, uriTemplate *url.URL, p
 				pathParams = []*cli.Param{}
 			}
 			pathParams = append(pathParams, param)
+			pathSchemas = append(pathSchemas, schema)
 		case "query":
 			if queryParams == nil {
 				queryParams = []*cli.Param{}
 			}
 			queryParams = append(queryParams, param)
+			querySchemas = append(querySchemas, schema)
 		case "header":
 			if headerParams == nil {
 				headerParams = []*cli.Param{}
 			}
 			headerParams = append(headerParams, param)
+			headerSchemas = append(headerSchemas, schema)
 		}
 	}
 
 	aliases := getExtSlice(op.Extensions, ExtAliases, []string{})
 
 	name := casing.Kebab(op.OperationId)
+	if name == "" {
+		name = casing.Kebab(method + "-" + strings.Trim(uriTemplate.Path, "/"))
+	}
 	if override := getExt(op.Extensions, ExtName, ""); override != "" {
 		name = override
-	} else if oldName := slug.Make(op.OperationId); oldName != name {
+	} else if oldName := slug.Make(op.OperationId); oldName != "" && oldName != name {
 		// For backward-compatibility, add the old naming scheme as an alias
 		// if it is different. See https://github.com/danielgtaylor/restish/issues/29
 		// for additional context; we prefer kebab casing for readability.
@@ -304,6 +326,25 @@ func openapiOperation(cmd *cobra.Command, method string, uriTemplate *url.URL, p
 	desc := getExt(op.Extensions, ExtDescription, op.Description)
 	hidden := getExt(op.Extensions, ExtHidden, false)
 
+	if len(pathParams) > 0 {
+		desc += "\n## Argument Schema:\n```schema\n{\n"
+		for i, p := range pathParams {
+			desc += "  " + p.OptionName() + ": " + paramSchema(p, pathSchemas[i]) + "\n"
+		}
+		desc += "}\n```\n"
+	}
+
+	if len(queryParams) > 0 || len(headerParams) > 0 {
+		desc += "\n## Option Schema:\n```schema\n{\n"
+		for i, p := range queryParams {
+			desc += "  --" + p.OptionName() + ": " + paramSchema(p, querySchemas[i]) + "\n"
+		}
+		for i, p := range headerParams {
+			desc += "  --" + p.OptionName() + ": " + paramSchema(p, headerSchemas[i]) + "\n"
+		}
+		desc += "}\n```\n"
+	}
+
 	mediaType := ""
 	var examples []string
 	if op.RequestBody != nil {
@@ -517,7 +558,17 @@ func loadOpenAPI3(cfg Resolver, cmd *cobra.Command, location *url.URL, resp *htt
 		return cli.API{}, err
 	}
 
-	doc, err := libopenapi.NewDocument(data)
+	config := datamodel.NewOpenDocumentConfiguration()
+	schemeLower := strings.ToLower(location.Scheme)
+	if schemeLower == "http" || schemeLower == "https" {
+		// Set the base URL to resolve relative references.
+		config.BaseURL = &url.URL{Scheme: location.Scheme, Host: location.Host, Path: path.Dir(location.Path)}
+	} else {
+		// Set the base local directory path to resolve relative references.
+		config.BasePath = path.Dir(location.Path)
+	}
+
+	doc, err := libopenapi.NewDocumentWithConfiguration(data, config)
 	if err != nil {
 		return cli.API{}, err
 	}
@@ -542,7 +593,7 @@ func loadOpenAPI3(cfg Resolver, cmd *cobra.Command, location *url.URL, resp *htt
 	}
 
 	// See if this server has any base path prefix we need to account for.
-	basePath, err := getBasePath(location, model.Servers)
+	basePath, err := getBasePath(cfg.GetBase(), model.Servers)
 	if err != nil {
 		return cli.API{}, err
 	}
@@ -554,7 +605,7 @@ func loadOpenAPI3(cfg Resolver, cmd *cobra.Command, location *url.URL, resp *htt
 				continue
 			}
 
-			resolved, err := cfg.Resolve(basePath + uri)
+			resolved, err := cfg.Resolve(strings.TrimSuffix(basePath, "/") + uri)
 			if err != nil {
 				return cli.API{}, err
 			}
@@ -716,6 +767,10 @@ type loader struct {
 	base     *url.URL
 }
 
+func (l *loader) GetBase() *url.URL {
+	return l.base
+}
+
 func (l *loader) Resolve(relURI string) (*url.URL, error) {
 	parsed, err := url.Parse(relURI)
 	if err != nil {
